from dataclasses import fields


def complete(a: object, b: object):
    """用a的值补全b缺失的值"""
    for f in fields(type(a)):
        b.__dict__[f.name] = b.__dict__[f.name] or a.__dict__[f.name]
    return b



from copy import deepcopy
from dataclasses import dataclass
from typing import Final, TypeVar, TypeAlias, Callable
from typing import Protocol

Self = TypeVar('Self', bound='PRecord')


class PRecord(Protocol):
    """数据库记录"""
    id: int
    """记录ID"""

    def clone(self: Self) -> Self:
        """克隆记录"""
        return deepcopy(self)


RecordFilter: TypeAlias = Callable[[PRecord], bool]
"""记录过滤器"""


@dataclass
class DemoRecord(PRecord):
    """用于演示/测试的记录"""
    id: int
    name: str


R1: Final[DemoRecord] = DemoRecord(1, 'group1')
R2: Final[DemoRecord] = DemoRecord(2, 'group2')

GROUP_DIR: Final[str] = '/opt/ias/project/shtm/node/n1/db/group'



from pathlib import Path
from typing import TypeVar, Type

from jcx.db.precord import PRecord, RecordFilter
from jcx.sys.fs import StrPath
from jcx.text.txt_json import load_json
from rustshed import Option, Null

R = TypeVar("R", bound=PRecord)


def load_list(record_type: Type[R], folder: StrPath, filter_: Option[RecordFilter] = Null) -> list[R]:
    """加载记录到列表"""
    records: list[R] = []

    folder = Path(folder)
    if not folder.is_dir():
        return records

    for f in folder.glob('*.json'):
        r = load_json(f, record_type)
        if filter_.is_some() and not filter_.unwrap()(r):
            continue
        records.append(r)
    return records


def load_dict(record_type: Type[R], folder: StrPath, filter_: Option[RecordFilter] = Null) -> dict[int, R]:
    """加载记录到字典"""
    rs = load_list(record_type, folder, filter_)
    return {r.id: r for r in rs}



from dataclasses import dataclass


@dataclass
class Counter:
    """计数器"""
    count: int = 0
    """计数值"""

    def __str__(self):
        return 'Counter(%d)' % self.count

    def get(self) -> int:
        """获取计数值"""
        return self.count

    def get_inc(self) -> int:
        """获取计数值并更新计数器"""
        t = self.get()
        self.count += 1
        return t


@dataclass
class CountTimer:
    """计数闹钟"""
    interval: int
    """间隔"""
    count: int = 0
    """计数值"""

    def __str__(self):
        return 'CountTimer(%d,%d)' % (self.interval, self.count)

    def inc(self):
        """更新计数器"""
        self.count += 1

    def check(self) -> bool:
        """计数器是否到规定的间隔"""
        return self.count >= self.interval

    def inc_check(self) -> bool:
        """计数器累计，并检查是否到规定的间隔"""
        self.inc()
        return self.check()

    def auto_check(self) -> bool:
        """计数器累计，并检查是否到规定的间隔，到规定时间则自动复位"""
        ok = self.inc_check()
        if ok:
            self.reset()
        return ok

    def reset(self):
        """计数器复位"""
        self.count = 0
